/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "util/util.h"
#include <stdio.h>

#if defined HAVE_STRING_H || defined _LIBC
# include <string.h>
#else
# include <strings.h>
#endif

static char* juggle (char *buf, unsigned long l, char c);

/* This function should solve the problem once and for all. */
/* Surely someone will think of a more beautiful solution.  */
const char* l2si (unsigned long l)
{
  static char buf[8];
  
  /* Check if SI units for bytes really use 1000 as multiplicator. */
  /* (Last I checked, they are, in the U.S.) */
  /* bytes range from 0 - 9999       */
  /* k     range from 10.00k - 9999k */
  /* M     range from 10.00M - 9999M */
  /* G     range from 10.00G - 9999G */
  /* T     range from 10.00T - 9999T */
  /* P     range from 10.00P - 9999P */
  
  if (l < 10000) {
  	sprintf (buf, "%4d", (int)l);
  	return buf;
  }

  if (l < 10000000L)
    return juggle (buf, l, 'k'); 
  l /= 1000L;
  if (l < 10000000L)
    return juggle (buf, l, 'M'); 
  l /= 1000L;
  if (l < 10000000L)
    return juggle (buf, l, 'G'); 
  l /= 1000L;
  if (l < 10000000L)
    return juggle (buf, l, 'T'); 
  l /= 1000L;
  if (l < 10000000L)
    return juggle (buf, l, 'P'); 
    
  fprintf (stderr, "l2si: Argument too big!\n");
  exit (1);
}

static char* juggle (char *buf, unsigned long l, char c)
{
  if (l < 100000L) {
  	sprintf (buf, "%d %c", (int)(l/10), c);
  	buf[4] = buf[3];
  	buf[3] = buf[2];
  	buf[2] = '.';
  	return buf;
  }
  if (l < 1000000L) {
  	sprintf (buf, "%d %c", (int)(l/100), c);
  	buf[4] = buf[3];
  	buf[3] = '.';
  	return buf;
  }
  sprintf (buf, "%d%c", (int)(l/1000), c);
  return buf;
}

